@@ -166,8 +166,7 @@ _unbind_buffer (struct SrcShader *src)
 }
 
 static gboolean
-_src_shader_init (gpointer impl, GstGLContext * context,
-    const GstVideoInfo * v_info)
+_src_shader_init (gpointer impl, GstGLContext * context, GstVideoInfo * v_info)
 {
   struct SrcShader *src = impl;
   const GstGLFuncs *gl = context->gl_vtable;
@@ -223,8 +222,7 @@ _src_shader_fill_bound_fbo (gpointer impl)
 
   if (gl->GenVertexArrays)
     gl->BindVertexArray (0);
-  else
-    _unbind_buffer (src);
+  _unbind_buffer (src);
 
   gst_gl_context_clear_shader (src->base.context);
 
@@ -266,6 +264,9 @@ static const gchar *smpte_vertex_src =
     "}";
 
 static const gchar *smpte_fragment_src =
+    "#ifdef GL_ES\n"
+    "precision mediump float;\n"
+    "#endif\n"
     "varying vec4 color;\n"
     "void main()\n"
     "{\n"
@@ -281,7 +282,10 @@ static const gchar *snow_vertex_src =
     "   out_uv = position.xy;\n"
     "}";
 
-static const gchar *snow_fragment_src =
+static const gchar *snow_fragment_src = 
+    "#ifdef GL_ES\n"
+    "precision mediump float;\n"
+    "#endif\n"
     "uniform float time;\n"
     "varying vec2 out_uv;\n"
     "\n"
@@ -315,15 +319,13 @@ _src_smpte_new (GstGLTestSrc * test)
 }
 
 static gboolean
-_src_smpte_init (gpointer impl, GstGLContext * context,
-    const GstVideoInfo * v_info)
+_src_smpte_init (gpointer impl, GstGLContext * context, GstVideoInfo * v_info)
 {
   struct SrcSMPTE *src = impl;
   struct XYZWRGB *coord;
   gushort *plane_indices;
   GError *error = NULL;
   int color_idx = 0;
-  const gchar *frags[2];
   int i;
 
   src->base.base.context = context;
@@ -444,29 +446,20 @@ _src_smpte_init (gpointer impl, GstGLContext * context,
 
   if (src->color_shader)
     gst_object_unref (src->color_shader);
-
-  frags[0] =
-      gst_gl_shader_string_get_highest_precision (context,
-      GST_GLSL_VERSION_NONE,
-      GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY);
-  frags[1] = smpte_fragment_src;
-
   src->color_shader = gst_gl_shader_new_link_with_stages (context, &error,
       gst_glsl_stage_new_with_string (context, GL_VERTEX_SHADER,
           GST_GLSL_VERSION_NONE,
           GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
           smpte_vertex_src),
-      gst_glsl_stage_new_with_strings (context, GL_FRAGMENT_SHADER,
+      gst_glsl_stage_new_with_string (context, GL_FRAGMENT_SHADER,
           GST_GLSL_VERSION_NONE,
-          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY, 2,
-          frags), NULL);
+          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
+          smpte_fragment_src), NULL);
   if (!src->color_shader) {
     GST_ERROR_OBJECT (src->base.base.src, "%s", error->message);
     return FALSE;
   }
 
-  frags[1] = snow_fragment_src;
-
   if (src->snow_shader)
     gst_object_unref (src->snow_shader);
   src->snow_shader = gst_gl_shader_new_link_with_stages (context, &error,
@@ -474,10 +467,10 @@ _src_smpte_init (gpointer impl, GstGLContext * context,
           GST_GLSL_VERSION_NONE,
           GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
           snow_vertex_src),
-      gst_glsl_stage_new_with_strings (context, GL_FRAGMENT_SHADER,
+      gst_glsl_stage_new_with_string (context, GL_FRAGMENT_SHADER,
           GST_GLSL_VERSION_NONE,
-          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY, 2,
-          frags), NULL);
+          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
+          snow_fragment_src), NULL);
   if (!src->snow_shader) {
     GST_ERROR_OBJECT (src->base.base.src, "%s", error->message);
     return FALSE;
@@ -517,7 +510,6 @@ _src_smpte_fill_bound_fbo (gpointer impl)
 {
   struct SrcSMPTE *src = impl;
   gint attr_color_position = -1;
-  GstGLBaseSrc *bsrc = GST_GL_BASE_SRC (src->base.base.src);
 
   src->base.n_attributes = 2;
   if (src->base.shader)
@@ -538,7 +530,7 @@ _src_smpte_fill_bound_fbo (gpointer impl)
   src->base.index_offset = (N_QUADS - 1) * 6 * sizeof (gushort);
   gst_gl_shader_use (src->snow_shader);
   gst_gl_shader_set_uniform_1f (src->snow_shader, "time",
-      (gfloat) bsrc->running_time / GST_SECOND);
+      (gfloat) src->base.base.src->running_time / GST_SECOND);
   if (!_src_shader_fill_bound_fbo (impl))
     return FALSE;
   src->attr_snow_position = src->base.attributes[0].location;
@@ -597,7 +589,7 @@ _src_uni_color_new (GstGLTestSrc * test)
 
 static gboolean
 _src_uni_color_init (gpointer impl, GstGLContext * context,
-    const GstVideoInfo * v_info)
+    GstVideoInfo * v_info)
 {
   struct SrcUniColor *src = impl;
 
@@ -691,6 +683,9 @@ static const gchar *checkers_vertex_src = "attribute vec4 position;\n"
     "}";
 
 static const gchar *checkers_fragment_src =
+    "#ifdef GL_ES\n"
+    "precision mediump float;\n"
+    "#endif\n"
     "uniform float checker_width;\n"
     "uniform float width;\n"
     "uniform float height;\n"
@@ -714,20 +709,13 @@ struct SrcCheckers
 
 static gboolean
 _src_checkers_init (gpointer impl, GstGLContext * context,
-    const GstVideoInfo * v_info)
+    GstVideoInfo * v_info)
 {
   struct SrcCheckers *src = impl;
   GError *error = NULL;
-  const gchar *frags[2];
 
   src->base.base.context = context;
 
-  frags[0] =
-      gst_gl_shader_string_get_highest_precision (context,
-      GST_GLSL_VERSION_NONE,
-      GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY);
-  frags[1] = checkers_fragment_src;
-
   if (src->base.shader)
     gst_object_unref (src->base.shader);
   src->base.shader = gst_gl_shader_new_link_with_stages (context, &error,
@@ -735,10 +723,10 @@ _src_checkers_init (gpointer impl, GstGLContext * context,
           GST_GLSL_VERSION_NONE,
           GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
           checkers_vertex_src),
-      gst_glsl_stage_new_with_strings (context, GL_FRAGMENT_SHADER,
+      gst_glsl_stage_new_with_string (context, GL_FRAGMENT_SHADER,
           GST_GLSL_VERSION_NONE,
-          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY, 2,
-          frags), NULL);
+          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
+          checkers_fragment_src), NULL);
   if (!src->base.shader) {
     GST_ERROR_OBJECT (src->base.base.src, "%s", error->message);
     return FALSE;
@@ -815,21 +803,13 @@ SRC_CHECKERS (4);
 SRC_CHECKERS (8);
 
 static gboolean
-_src_snow_init (gpointer impl, GstGLContext * context,
-    const GstVideoInfo * v_info)
+_src_snow_init (gpointer impl, GstGLContext * context, GstVideoInfo * v_info)
 {
   struct SrcShader *src = impl;
   GError *error = NULL;
-  const gchar *frags[2];
 
   src->base.context = context;
 
-  frags[0] =
-      gst_gl_shader_string_get_highest_precision (context,
-      GST_GLSL_VERSION_NONE,
-      GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY);
-  frags[1] = snow_fragment_src;
-
   if (src->shader)
     gst_object_unref (src->shader);
   src->shader = gst_gl_shader_new_link_with_stages (context, &error,
@@ -837,10 +817,10 @@ _src_snow_init (gpointer impl, GstGLContext * context,
           GST_GLSL_VERSION_NONE,
           GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
           snow_vertex_src),
-      gst_glsl_stage_new_with_strings (context, GL_FRAGMENT_SHADER,
+      gst_glsl_stage_new_with_string (context, GL_FRAGMENT_SHADER,
           GST_GLSL_VERSION_NONE,
-          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY, 2,
-          frags), NULL);
+          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
+          snow_fragment_src), NULL);
   if (!src->shader) {
     GST_ERROR_OBJECT (src->base.src, "%s", error->message);
     return FALSE;
@@ -867,14 +847,13 @@ static gboolean
 _src_snow_fill_bound_fbo (gpointer impl)
 {
   struct SrcShader *src = impl;
-  GstGLBaseSrc *bsrc = GST_GL_BASE_SRC (src->base.src);
 
   g_return_val_if_fail (src->base.context, FALSE);
   g_return_val_if_fail (src->shader, FALSE);
 
   gst_gl_shader_use (src->shader);
   gst_gl_shader_set_uniform_1f (src->shader, "time",
-      (gfloat) bsrc->running_time / GST_SECOND);
+      (gfloat) src->base.src->running_time / GST_SECOND);
 
   return _src_shader_fill_bound_fbo (impl);
 }
@@ -921,7 +900,10 @@ static const gchar *mandelbrot_vertex_src = "attribute vec4 position;\n"
     "  fractal_position *= 2.5;\n"
     "}";
 
-static const gchar *mandelbrot_fragment_src =
+static const gchar *mandelbrot_fragment_src = 
+    "#ifdef GL_ES\n"
+    "precision mediump float;\n"
+    "#endif\n"
     "uniform float time;\n"
     "varying vec2 fractal_position;\n"
     "const vec4 K = vec4(1.0, 0.66, 0.33, 3.0);\n"
@@ -955,20 +937,13 @@ static const gchar *mandelbrot_fragment_src =
 
 static gboolean
 _src_mandelbrot_init (gpointer impl, GstGLContext * context,
-    const GstVideoInfo * v_info)
+    GstVideoInfo * v_info)
 {
   struct SrcShader *src = impl;
   GError *error = NULL;
-  const gchar *frags[2];
 
   src->base.context = context;
 
-  frags[0] =
-      gst_gl_shader_string_get_highest_precision (context,
-      GST_GLSL_VERSION_NONE,
-      GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY);
-  frags[1] = mandelbrot_fragment_src;
-
   if (src->shader)
     gst_object_unref (src->shader);
   src->shader = gst_gl_shader_new_link_with_stages (context, &error,
@@ -976,10 +951,10 @@ _src_mandelbrot_init (gpointer impl, GstGLContext * context,
           GST_GLSL_VERSION_NONE,
           GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
           mandelbrot_vertex_src),
-      gst_glsl_stage_new_with_strings (context, GL_FRAGMENT_SHADER,
+      gst_glsl_stage_new_with_string (context, GL_FRAGMENT_SHADER,
           GST_GLSL_VERSION_NONE,
-          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY, 2,
-          frags), NULL);
+          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
+          mandelbrot_fragment_src), NULL);
   if (!src->shader) {
     GST_ERROR_OBJECT (src->base.src, "%s", error->message);
     return FALSE;
@@ -1012,14 +987,13 @@ static gboolean
 _src_mandelbrot_fill_bound_fbo (gpointer impl)
 {
   struct SrcShader *src = impl;
-  GstGLBaseSrc *bsrc = GST_GL_BASE_SRC (src->base.src);
 
   g_return_val_if_fail (src->base.context, FALSE);
   g_return_val_if_fail (src->shader, FALSE);
 
   gst_gl_shader_use (src->shader);
   gst_gl_shader_set_uniform_1f (src->shader, "time",
-      (gfloat) bsrc->running_time / GST_SECOND);
+      (gfloat) src->base.src->running_time / GST_SECOND);
 
   return _src_shader_fill_bound_fbo (impl);
 }
@@ -1066,6 +1040,9 @@ static const gchar *circular_vertex_src =
     "}";
 
 static const gchar *circular_fragment_src =
+    "#ifdef GL_ES\n"
+    "precision mediump float;\n"
+    "#endif\n"
     "uniform float aspect_ratio;\n"
     "varying vec2 uv;\n"
     "#define PI 3.14159265\n"
@@ -1083,20 +1060,13 @@ static const gchar *circular_fragment_src =
 
 static gboolean
 _src_circular_init (gpointer impl, GstGLContext * context,
-    const GstVideoInfo * v_info)
+    GstVideoInfo * v_info)
 {
   struct SrcShader *src = impl;
   GError *error = NULL;
-  const gchar *frags[2];
 
   src->base.context = context;
 
-  frags[0] =
-      gst_gl_shader_string_get_highest_precision (context,
-      GST_GLSL_VERSION_NONE,
-      GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY);
-  frags[1] = circular_fragment_src;
-
   if (src->shader)
     gst_object_unref (src->shader);
   src->shader = gst_gl_shader_new_link_with_stages (context, &error,
@@ -1104,10 +1074,10 @@ _src_circular_init (gpointer impl, GstGLContext * context,
           GST_GLSL_VERSION_NONE,
           GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
           circular_vertex_src),
-      gst_glsl_stage_new_with_strings (context, GL_FRAGMENT_SHADER,
+      gst_glsl_stage_new_with_string (context, GL_FRAGMENT_SHADER,
           GST_GLSL_VERSION_NONE,
-          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY, 2,
-          frags), NULL);
+          GST_GLSL_PROFILE_ES | GST_GLSL_PROFILE_COMPATIBILITY,
+          circular_fragment_src), NULL);
   if (!src->shader) {
     GST_ERROR_OBJECT (src->base.src, "%s", error->message);
     return FALSE;
