@@ -22,24 +22,21 @@
 
 /**
  * SECTION:gstrtcpbuffer
+ * @title: GstRTCPBuffer
  * @short_description: Helper methods for dealing with RTCP buffers
  * @see_also: #GstRTPBasePayload, #GstRTPBaseDepayload, #gstrtpbuffer
  *
  * Note: The API in this module is not yet declared stable.
  *
- * <refsect2>
- * <para>
- * The GstRTPCBuffer helper functions makes it easy to parse and create regular 
+ * The GstRTPCBuffer helper functions makes it easy to parse and create regular
  * #GstBuffer objects that contain compound RTCP packets. These buffers are typically
  * of 'application/x-rtcp' #GstCaps.
- * </para>
- * <para>
+ *
  * An RTCP buffer consists of 1 or more #GstRTCPPacket structures that you can
  * retrieve with gst_rtcp_buffer_get_first_packet(). #GstRTCPPacket acts as a pointer
  * into the RTCP buffer; you can move to the next packet with
  * gst_rtcp_packet_move_to_next().
- * </para>
- * </refsect2>
+ *
  */
 
 #include <string.h>
@@ -82,7 +79,7 @@ gst_rtcp_buffer_new_take_data (gpointer data, guint len)
  * Returns: A newly allocated buffer with a copy of @data and of size @len.
  */
 GstBuffer *
-gst_rtcp_buffer_new_copy_data (gpointer data, guint len)
+gst_rtcp_buffer_new_copy_data (gconstpointer data, guint len)
 {
   return gst_rtcp_buffer_new_take_data (g_memdup (data, len), len);
 }
@@ -497,7 +494,7 @@ end:
  * @type: the #GstRTCPType of the new packet
  * @packet: pointer to new packet
  *
- * Add a new packet of @type to @rtcp. @packet will point to the newly created 
+ * Add a new packet of @type to @rtcp. @packet will point to the newly created
  * packet.
  *
  * Returns: %TRUE if the packet could be created. This function returns %FALSE
@@ -549,6 +546,9 @@ gst_rtcp_buffer_add_packet (GstRTCPBuffer * rtcp, GstRTCPType type,
     case GST_RTCP_TYPE_PSFB:
       len = 12;
       break;
+    case GST_RTCP_TYPE_XR:
+      len = 4;
+      break;
     default:
       goto unknown_type;
   }
@@ -677,7 +677,7 @@ gst_rtcp_packet_get_count (GstRTCPPacket * packet)
  * gst_rtcp_packet_get_length:
  * @packet: a valid #GstRTCPPacket
  *
- * Get the length field of @packet. This is the length of the packet in 
+ * Get the length field of @packet. This is the length of the packet in
  * 32-bit words minus one.
  *
  * Returns: The length field of @packet.
@@ -694,11 +694,11 @@ gst_rtcp_packet_get_length (GstRTCPPacket * packet)
 /**
  * gst_rtcp_packet_sr_get_sender_info:
  * @packet: a valid SR #GstRTCPPacket
- * @ssrc: result SSRC
- * @ntptime: result NTP time
- * @rtptime: result RTP time
- * @packet_count: result packet count
- * @octet_count: result octet count
+ * @ssrc: (out): result SSRC
+ * @ntptime: (out): result NTP time
+ * @rtptime: (out): result RTP time
+ * @packet_count: (out): result packet count
+ * @octet_count: (out): result octet count
  *
  * Parse the SR sender info and store the values.
  */
@@ -737,7 +737,7 @@ gst_rtcp_packet_sr_get_sender_info (GstRTCPPacket * packet, guint32 * ssrc,
 /**
  * gst_rtcp_packet_sr_set_sender_info:
  * @packet: a valid SR #GstRTCPPacket
- * @ssrc: the SSRC 
+ * @ssrc: the SSRC
  * @ntptime: the NTP time
  * @rtptime: the RTP time
  * @packet_count: the packet count
@@ -847,13 +847,13 @@ gst_rtcp_packet_get_rb_count (GstRTCPPacket * packet)
  * gst_rtcp_packet_get_rb:
  * @packet: a valid SR or RR #GstRTCPPacket
  * @nth: the nth report block in @packet
- * @ssrc: result for data source being reported
- * @fractionlost: result for fraction lost since last SR/RR
- * @packetslost: result for the cumululative number of packets lost
- * @exthighestseq: result for the extended last sequence number received
- * @jitter: result for the interarrival jitter
- * @lsr: result for the last SR packet from this source
- * @dlsr: result for the delay since last SR packet
+ * @ssrc: (out): result for data source being reported
+ * @fractionlost: (out): result for fraction lost since last SR/RR
+ * @packetslost: (out): result for the cumululative number of packets lost
+ * @exthighestseq: (out): result for the extended last sequence number received
+ * @jitter: (out): result for the interarrival jitter
+ * @lsr: (out): result for the last SR packet from this source
+ * @dlsr: (out): result for the delay since last SR packet
  *
  * Parse the values of the @nth report block in @packet and store the result in
  * the values.
@@ -993,7 +993,8 @@ gst_rtcp_packet_add_rb (GstRTCPPacket * packet, guint32 ssrc,
 
   GST_WRITE_UINT32_BE (data, ssrc);
   data += 4;
-  GST_WRITE_UINT32_BE (data, (fractionlost << 24) | (packetslost & 0xffffff));
+  GST_WRITE_UINT32_BE (data,
+      ((guint32) fractionlost << 24) | (packetslost & 0xffffff));
   data += 4;
   GST_WRITE_UINT32_BE (data, exthighestseq);
   data += 4;
@@ -1043,7 +1044,7 @@ gst_rtcp_packet_set_rb (GstRTCPPacket * packet, guint nth, guint32 ssrc,
 
 
 /**
- * gst_rtcp_packet_set_profile_specific_ext:
+ * gst_rtcp_packet_add_profile_specific_ext:
  * @packet: a valid SR or RR #GstRTCPPacket
  * @data: (array length=len) (transfer none): profile-specific data
  * @len: length of the profile-specific data in bytes
@@ -1746,7 +1747,7 @@ no_space:
 /**
  * gst_rtcp_packet_bye_add_ssrcs:
  * @packet: a valid BYE #GstRTCPPacket
- * @ssrc: an array of SSRCs to add
+ * @ssrc: (array length=len) (transfer none): an array of SSRCs to add
  * @len: number of elements in @ssrc
  *
  * Adds @len SSRCs in @ssrc to BYE @packet.
